/*
	Copyright 2007-2009 - Marc Dumoulin

	This file is part of Open-Maker software.

	Open-Maker software is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	Open-Maker software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/
#include "qlifegraphicsview.h"

QLifeGraphicsView::QLifeGraphicsView(int amountWidth,int amountHeight,int width,int height) {
	QSettings settings(qApp->organizationName(),qApp->applicationName());
	if (settings.contains("a_color"))
		m_activeColor = settings.value("a_color").value<QColor>();
	else m_activeColor = QColor("darkseagreen");
	if (settings.contains("na_color"))
		m_inactiveColor = settings.value("na_color").value<QColor>();
	else m_inactiveColor = QColor("white");
	
	m_amountWidth = amountWidth;
	m_amountHeight = amountHeight;
	m_width = width;
	m_height = height;
	
	m_leftMousePressed = false;
	m_rightMousePressed = false;
	m_scene = new QGraphicsScene;
	
	setScene(m_scene);
	setAlignment(Qt::AlignLeft | Qt::AlignTop);
	setFixedSize((m_amountWidth*m_width)+3,(m_amountHeight*m_height)+3);
	setBackgroundRole(QPalette::Base);
	setAutoFillBackground(true);
	setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	
	for (int x = 0; x < m_amountWidth; x++) {
		QList<QGraphicsRectItem*> line;
		QList<bool> stateLine;
		for (int y = 0; y < m_amountHeight; y++) {
			line << m_scene->addRect(m_width*x,m_height*y,m_width,m_height,QPen(),QBrush(m_inactiveColor,Qt::SolidPattern));
			stateLine << false;
		}
		m_grid << line;
		m_state << stateLine;
	}
}

QLifeGraphicsView::~QLifeGraphicsView() {
	delete m_scene;
}

QColor QLifeGraphicsView::activeColor() {
	return m_activeColor;
}

QColor QLifeGraphicsView::inactiveColor() {
	return m_inactiveColor;
}

void QLifeGraphicsView::newActiveColor() {
	QColor color = QColorDialog::getColor(m_activeColor);
	if (color.isValid() && color != m_activeColor) {
		m_activeColor = color;
		for (int y = 0; y < m_amountHeight; y++)
			for (int x = 0; x < m_amountWidth; x++)
				setState(x,y,m_state[x][y]);
	}
}

void QLifeGraphicsView::newInactiveColor() {
	QColor color = QColorDialog::getColor(m_inactiveColor);
	if (color.isValid() && color != m_inactiveColor) {
		m_inactiveColor = color;
		for (int y = 0; y < m_amountHeight; y++)
			for (int x = 0; x < m_amountWidth; x++)
				setState(x,y,m_state[x][y]);
	}
}

void QLifeGraphicsView::setDefaultColor() {
	m_activeColor = QColor("darkseagreen");
	m_inactiveColor = QColor("white");
	for (int y = 0; y < m_amountHeight; y++)
		for (int x = 0; x < m_amountWidth; x++)
			setState(x,y,m_state[x][y]);
}

void QLifeGraphicsView::clear() {
	for (int y = 0; y < m_amountHeight; y++)
		for (int x = 0; x < m_amountWidth; x++)
			setState(x,y,false);	
}

QList<QList<bool> >& QLifeGraphicsView::state() {
	return m_state;
}

void QLifeGraphicsView::setState(int x,int y,bool s) {
	if (s) m_grid[x][y]->setBrush(QBrush(m_activeColor,Qt::SolidPattern));
	else m_grid[x][y]->setBrush(QBrush(m_inactiveColor,Qt::SolidPattern));
	m_state[x][y] = s;
}

int QLifeGraphicsView::width() {
	return m_amountWidth;
}

int QLifeGraphicsView::height() {
	return m_amountHeight;	
}

void QLifeGraphicsView::mouseMoveEvent(QMouseEvent* event) {
	if (!m_leftMousePressed && !m_rightMousePressed) return;
	
	QGraphicsRectItem* item = getNode(event->pos());
	if (item == 0) return;
	
	if (m_leftMousePressed)
		item->setBrush(QBrush(m_activeColor,Qt::SolidPattern));
	if (m_rightMousePressed)
		item->setBrush(QBrush(m_inactiveColor,Qt::SolidPattern));
	
	bool* state = getNodeState(event->pos());
	*state = m_leftMousePressed;
}

void QLifeGraphicsView::mousePressEvent(QMouseEvent* event) {
	if (event->button() == Qt::LeftButton)
		m_leftMousePressed = true;
	if (event->button() == Qt::RightButton)
		m_rightMousePressed = true;
	
	QGraphicsRectItem* item = getNode(event->pos());
	if (item == 0) return;
	
	if (m_leftMousePressed)
		item->setBrush(QBrush(m_activeColor,Qt::SolidPattern));
	if (m_rightMousePressed)
		item->setBrush(QBrush(m_inactiveColor,Qt::SolidPattern));
	
	bool* state = getNodeState(event->pos());
	*state = m_leftMousePressed;
	
}

void QLifeGraphicsView::mouseReleaseEvent(QMouseEvent* event) {
	if (event->button() == Qt::LeftButton)
		m_leftMousePressed = false;
	if (event->button() == Qt::RightButton)
		m_rightMousePressed = false;
}

QGraphicsRectItem* QLifeGraphicsView::getNode(const QPoint& pos) {
	int x = pos.x();
	int y = pos.y();
	if (x <= 0 || x >= m_width*m_amountWidth || y <= 0 || y >= m_height*m_amountHeight)
		return 0;
	x = x/m_width;
	y = y/m_height;
	return m_grid[x][y];
}

bool* QLifeGraphicsView::getNodeState(const QPoint& pos) {
	int x = pos.x();
	int y = pos.y();
	if (x <= 0 || x >= m_width*m_amountWidth || y <= 0 || y >= m_height*m_amountHeight)
		return 0;
	x = x/m_width;
	y = y/m_height;
	return &m_state[x][y];
}

